﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace TransparentForm
{
    public class TransparentForm : Form
    {
        private bool fFlicker = true;

        private bool fDrag;
        private Point pointDown;

        private Color colorKey = Color.FromArgb(255, 128, 128, 128);

        public TransparentForm()
        {
            Size = new Size(320, 320);
            FormBorderStyle = FormBorderStyle.None;
            ShowInTaskbar = false;

            TransparencyKey = colorKey;

            TopMost = true;

            SetStyle(ControlStyles.Opaque, true);
            SetStyle(ControlStyles.UserPaint, true);
            SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            SetStyle(ControlStyles.OptimizedDoubleBuffer, true);

            Show();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            Paint(g);
        }

        private void Paint(Graphics g)
        {
            g.CompositingMode = CompositingMode.SourceCopy;
            g.FillRectangle(new SolidBrush(colorKey), 0, 0, Width, Height);

            g.CompositingMode = CompositingMode.SourceOver;
            GraphicsPath gp = new GraphicsPath();
            gp.AddEllipse(0, 0, Width, Height);
            gp.AddEllipse(30 * Width / 120, 40 * Height / 120, 15 * Width / 120, 15 * Height / 120);
            gp.AddEllipse(80 * Width / 120, 40 * Height / 120, 15 * Width / 120, 15 * Height / 120);
            gp.AddRectangle(new Rectangle(30 * Width / 120, 80 * Height / 120, 60 * Width / 120, 15 * Height / 120));
            g.FillPath(Brushes.Yellow, gp);
            g.DrawLine(Pens.Blue, 0, 0, Width - 1, Height - 1);
            g.DrawLine(Pens.Blue, Width - 1, 0, 0, Height - 1);

            StringFormat sf = new StringFormat(StringFormat.GenericDefault);
            sf.Alignment = StringAlignment.Center;
            g.DrawString("Drag with left button to move\n"
                         + "Drag with right button to resize\n"
                         + "Click with middle button to enable/disable flickering",
                         new Font("Verdana", 8), Brushes.Black, 
                         new RectangleF(5, (float)60 * Height / 120, Width - 10, 50),
                         sf);

            g.DrawRectangle((fFlicker ? Pens.Black : new Pen(colorKey)), 0, 0, Width - 1, Height - 1);
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            fDrag = true;
            pointDown = e.Location;
        }

        protected override void OnMouseUp(MouseEventArgs e)
        {
            fDrag = false;
        }

        protected override void OnMouseClick(MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Middle) != 0)
            {
                fFlicker = !fFlicker;
                Invalidate();
            }
        }

        protected override void OnMouseMove(MouseEventArgs e)
        {
            if (fDrag)
            {
                if ((e.Button & MouseButtons.Left) != 0)
                {
                    Point point = Location;
                    point.Offset(e.X - pointDown.X, e.Y - pointDown.Y);
                    Location = point;
                }
                else
                {
                    Size size = Size;
                    size.Width += e.X - pointDown.X;
                    size.Height += e.Y - pointDown.Y;
                    this.Size = size;
                    Refresh();
                    pointDown = e.Location;
                }
            }
        }

        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new TransparentForm());
        }
    }
}